/**
 * COOS - Connected Objects Operating System (www.connectedobjects.org).
 *
 * Copyright (C) 2009 Telenor ASA and Tellu AS. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may also contact one of the following for additional information:
 * Telenor ASA, Snaroyveien 30, N-1331 Fornebu, Norway (www.telenor.no)
 * Tellu AS, Hagalokkveien 13, N-1383 Asker, Norway (www.tellu.no)
 */
package org.coos.plugin.logserver.impl;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;

import org.slf4j.LoggerFactory;

import ch.qos.logback.classic.Logger;
import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.classic.joran.JoranConfigurator;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.joran.spi.JoranException;
import ch.qos.logback.core.util.StatusPrinter;

/**
 * This class 
 * @author Robert Bj&aelig;rum, Tellu AS
 *
 */
public class LogbackServerNode {

	static Logger LOGGER = (Logger) LoggerFactory.getLogger(LogbackServerNode.class);
	LoggerContext lc;


	LogServerSM logServerSM;

	/**
	 * Constructor that uses the default LoggerContext.
	 * 
	 * @param logServerSM the LogServer state machine that will use this LogbackServerNode.
	 */
	public LogbackServerNode(LogServerSM logServerSM) {
		this.logServerSM = logServerSM;
		lc = (LoggerContext) LoggerFactory.getILoggerFactory();
	}

	/**
	 * Constructor that will create and configure LoggerContext based on configuration file.
	 * 
	 * @param logServerSM the LogServer state machine that will use this LogbackServerNode.
	 */
	public LogbackServerNode(LogServerSM logServerSM, InputStream configFileAsStream) throws Exception {
		this.logServerSM = logServerSM;
		lc = configure(configFileAsStream);
	}

	private LoggerContext configure(InputStream configFile) throws Exception {
		LoggerContext lc = null;
		try {
			JoranConfigurator configurator = new JoranConfigurator();
			lc = new LoggerContext();
			configurator.setContext(lc);
			configurator.doConfigure(configFile);
			StatusPrinter.print(lc);
			return lc;
		} catch (JoranException e) {
			e.printStackTrace();
			throw new Exception("Error parsing configuration for log server", e);
		}
	}

	public void logTheEvent(byte[] serializedEvent) {
		ILoggingEvent event;
		Logger remoteLogger;
		
		if (lc == null) {
			return;
		}

		try {
			/* Cast the remotely generated event */
			event = deserializeEvent(serializedEvent);

			/* Get a logger with name as when it was created */
			remoteLogger = lc.getLogger(event.getLoggerName());

			/* Log the event */
			if (remoteLogger.isEnabledFor(event.getLevel())) {
				remoteLogger.callAppenders(event);
			}

		} catch (Exception e) {
			LOGGER.error("Unexpected exception.", e);
		}
	}
	
	protected ILoggingEvent deserializeEvent(byte[] serializedEvent) {
		ILoggingEvent ev = null;
		try {
			ByteArrayInputStream bis = new ByteArrayInputStream(serializedEvent);
			ObjectInputStream inp = new ObjectInputStream(bis);
			ev = (ILoggingEvent) inp.readObject();
		} catch (IOException e) {
			e.printStackTrace();
			LOGGER.error("IOException caught when receiving remote logging event. Event will be ignored.", e);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			LOGGER.error("Failed to deserialize remote logging event. Event will be ignored.", e);
		}
		return ev;
	}

}
